#!/bin/bash
#Description: plugin agent install entry
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2023. All rights reserved.
#History: 2023-6-17 created
current_dir=$(cd $(dirname "$0"); pwd)
source "${current_dir}/utils.sh"
install_path="$1/DevKit"
INSTALL_LOG="${install_path}/devkitplugins/sys_diagnosis/deploy/sys_diagnosis_agent_install.log"
touch "${INSTALL_LOG}"
chmod 600 "${INSTALL_LOG}"
architecture=$(uname -m)
USER_NAME="devkit"

if [[ "${architecture}" != "aarch64" ]]; then
    echo "Not ARM machine, exit"
    exit "${ERROR}"
fi

EXIT_FLAG="TRUE" # 记录exit_clean状态防止进入死循环
trap "agent_exit_clean" EXIT

# 检查是否有安装残留并清理
function clean_residue() {
    echo "Clean sys_diagnosis residue"
    delete_devkit_lib
    delete_sudo_tools "${INSTALL_LOG}"
    local sys_diagnosis_install_path="${install_path}/devkitplugins/sys_diagnosis"
    if [ -e "${sys_diagnosis_install_path}" ]; then
        rm -rf "${sys_diagnosis_install_path}"
    fi
    echo "Clean sys_diagnosis residue success"
}


function agent_exit_clean() {
    exit_code="$?"
    if [ ${exit_code} != 0 ] && [[ ${EXIT_FLAG} == "TRUE" ]] ; then
        EXIT_FLAG="FALSE"
        clean_residue
        exit 1
    fi
}


function installed_check()
{
    if [ -e "${SUDO_FILE}" ]; then
        echo_and_log "Sys_diagnosis agent has already been deployed!" "${ERROR}" "${INSTALL_LOG}"
        exit "${ERROR}"
    fi
}


function init_variable() {
    if [[ ! -f "${CONFIG_FILE}" ]]; then
        echo_and_log "There is no configure file(${CONFIG_FILE}), please check!" "${ERROR}"
        exit "${ERROR}"
    fi
    os_type=$(grep -w OS_TYPE "${CONFIG_FILE}" | awk -F= '{if($2 !="") print $2}')
    os_version=$(grep -w OS_VERSION "${CONFIG_FILE}" | awk -F= '{if($2 !="") print $2}')
    INSTALL_PLUGIN_PATH="${install_path}/devkitplugins"
}


function agent_libcheck()
{
    echo_and_log "\nCheck agent Dependent Packages" "${DOING}" "${INSTALL_LOG}"
    package_all=("pre" "essential" "non_essential" )
    bash "${current_dir}/env_check.sh" "${install_path}" "agent"
    if [ "$?" -ne "${SUCCESS}" ]; then
        echo_and_log "Check install Dependent Packages Failed, Stop installing sys_diagnosis agent" "${ERROR}" "${INSTALL_LOG}"
        exit "${ERROR}"
    else
        echo_and_log "Check install Dependent Packages Success" "${SUCCESS}" "${INSTALL_LOG}"
    fi
}


function change_file_path_permission() {
    scripts=$(find ${INSTALL_PLUGIN_PATH}/sys_diagnosis/deploy -name "*\.sh")
    if [[ "${scripts}" ]]; then
        chown root:root ${scripts}
    fi
    if [[ -e "${INSTALL_PLUGIN_PATH}/sys_diagnosis" ]]; then
        chmod 700 $(find ${INSTALL_PLUGIN_PATH}/sys_diagnosis -type d)
        chmod 755 ${INSTALL_PLUGIN_PATH}/sys_diagnosis
    fi
    local deploy_path="${INSTALL_PLUGIN_PATH}/sys_diagnosis/deploy/"
    if [[ -e "${deploy_path}" ]]; then
        chown root:root "${deploy_path}"
        chown root:root "${INSTALL_PLUGIN_PATH}/sys_diagnosis"
        chown root:root "${INSTALL_PLUGIN_PATH}"
    fi
    # 所有so权限更改为550
    find ${INSTALL_PLUGIN_PATH}/sys_diagnosis/ -type f -name "*.so" -print0 | xargs -0 chmod 550 >> /dev/null 2>&1
    find ${INSTALL_PLUGIN_PATH}/sys_diagnosis/ -type f -name "*.so.*" -print0 | xargs -0 chmod 550 >> /dev/null 2>&1
}

function create_log_dir() {
    if [ -d "${install_path}/logs" ];then
        rm -rf ${install_path}/logs
    fi
    mkdir -p ${install_path}/logs
    chown "${USER_NAME}":"${USER_NAME}" ${install_path}/logs
    chmod 755 ${install_path}/logs
}


# 删除无用的文件
function rm_useless_files() {
    local install_path=$1
    local deploy_dir="${install_path}/devkitplugins/sys_diagnosis/deploy"
    if [ -e "${deploy_dir}" ]; then
        cd "${deploy_dir}"
        local rm_files=("agent_install_entry.sh" "install.sh" "env_check.sh" "uninstall.sh")
        for ((i=0;i<${#rm_files[@]};i++))
        do
            if [ -e "${rm_files[$i]}" ]; then
                rm -f "${rm_files[$i]}"
                echo_and_log "Remove ${rm_files[$i]} success" "${SUCCESS}" "${INSTALL_LOG}"
            else
                echo_and_log "Remove ${rm_files[$i]} failed, continue" "${DOING}" "${INSTALL_LOG}"
            fi
        done
        cd - 1>/dev/null
    fi
}


function main() {
    init_variable
    installed_check
    echo_and_log "Installing sys_diagnosis agent" "${DOING}" "${INSTALL_LOG}"
    agent_libcheck
    copy_sudo_files "${install_path}" "${INSTALL_LOG}" "agent"
    add_sudoers "${install_path}" "${INSTALL_LOG}"
    install_third_app
    create_log_dir
    change_file_path_permission
    echo_and_log "Installing sys_diagnosis agent success" "${SUCCESS}" "${INSTALL_LOG}"
    rm_useless_files "${install_path}"
    # 所有安装完成后，更改日志文件权限为只读
    chmod 400 "${INSTALL_LOG}"
}


main $@
